-- Title: Heirloom Quest Tracker
-- Author: LownIgnitus
-- Version: 1.0.3
-- Desc: Addon to track Misprinted Coins, Quest completions for free heiloom upgrades, and 1st heroic end boss kill of the day.

-- Globals
--local L = LibStub("AceLocale-3.0"):GetLocale("HeirloomQuestTracker")
HeirloomQuestTracker = LibStub("AceAddon-3.0"):NewAddon("HeirloomQuestTracker", "AceConsole-3.0", "AceEvent-3.0");

local coinID = 122618
local coinIcon = "Interface\\ICONS\\INV_Misc_Coin_18"

local textures = {}
textures.alliance = "|TInterface\\FriendsFrame\\PlusManz-Alliance:18|t"
textures.horde = "|TInterface\\FriendsFrame\\PlusManz-Horde:18|t"
textures.incomplete = "|TInterface\\GossipFrame\\ActiveQuestIcon:18|t"
textures.armorOne = "|TInterface\\ICONS\\INV_Icon_HeirloomToken_Armor01:18|t"
textures.armorTwo = "|TInterface\\ICONS\\INV_Icon_HeirloomToken_Armor02:18|t"
textures.weaponOne = "|TInterface\\ICONS\\INV_Icon_HeirloomToken_Weapon01:18|t"
textures.weaponTwo = "|TInterface\\ICONS\\INV_Icon_HeirloomToken_Weapon02:18|t"

SLASH_HEIRLOOMQUESTTRACKER1 = '/HQT' or '/hqt'

local addonName = "HeirloomQuestTracker";
local LDB = LibStub("LibDataBroker-1.1", true)
local LDBIcon = LDB and LibStub("LibDBIcon-1.0", true)
local LibQTip = LibStub('LibQTip-1.0')

local red = { r = 1.0, g = 0.2, b = 0.2 }
local blue = { r = 0.4, g = 0.4, b = 1.0 }
local green = { r = 0.2, g = 1.0, b = 0.2 }
local yellow = { r = 1.0, g = 1.0, b = 0.2 }
local gray = { r = 0.5, g = 0.5, b = 0.5 }
local black = { r = 0.0, g = 0.0, b = 0.0 }
local white = { r = 1.0, g = 1.0, b = 1.0 }
local frame

local HeirloomQuestTrackerLauncher = LDB:NewDataObject(addonName, {
		type = "data source",
		text = "Heirloom Quest Tracker",
		label = "HeirloomQuestTracker",
		tocname = "HeirloomQuestTracker",
		icon = coinIcon,
		OnClick = function(clickedframe, button)
			HeirloomQuestTracker:ShowOptions()
		end,
		OnEnter = function(self)
			frame = self
			HeirloomQuestTracker:ShowToolTip()
		end,
	})

local defaults = {
	realm = {
		characters = {
		},
	},
	global = {
		realms = {
			},
		MinimapButton = {
			hide = false,
		},
		displayOptions = {
			showMinimapButton = true,
		},
		toonOptions = {
			levelRestriction = true,
			minimumLevel = 100,
			removeInactive = true,
			inactivityThreshold = 28,
			include =3,
		},
	},
};

local options = {
	handler = HeirloomQuestTracker,
	type = "group",
	args = {
		generalOptions = {
			handler = HeirloomQuestTracker,
			type = "group",
			name = "General Options",
			desc = "",
			order = 10,
			args = {
				displayOptions = {
					type = "group",
					inline = true,
					name = "Display Options",
					desc = "",
					order = 1,
					args = {
						showMinimapButton = {
							type = "toggle",
							name = "Minimap Button",
							desc = "Toggles the display of the minimap button.",
							get = "IsShowMinimapButton",
							set = "ToggleMinimapButton",
							order = 1,
						},
					},
				},
			},
		},
		toonOptions = {
			handler = HeirloomQuestTracker,
			type = "group",
			name = "Toon Options",
			desc = "",
			order = 20,
			args = {
				includetoonOptions = {
					type = "group",
					inline = true,
					name = "Show Toons",
					desc = "",
					order = 1,
					args = {
						serverOptions = {
							type = "toggle",
							name = "On this Server",
							desc = "Show toons on this server.",
							get = function(info)
								return HeirloomQuestTracker.db.global.toonOptions.include == 2
							end,
							set = function(info, value)
								if value then
									HeirloomQuestTracker.db.global.toonOptions.include = 2
								else
									HeirloomQuestTracker.db.global.toonOptions.include = 1
								end
							end,
							order = 1,
						},
						accountOptions = {
							type = "toggle",
							name = "On this Account",
							desc = "Show Toons on this WoW account.",
							get = function(info)
								return HeirloomQuestTracker.db.global.toonOptions.include == 3
							end,
							set = function(info, value)
								if value then
									HeirloomQuestTracker.db.global.toonOptions.include = 3
								else
									HeirloomQuestTracker.db.global.toonOptions.include = 1
								end
							end,
							order = 2,
						},
					},
				},
				toonLevelOptions = {
					type = "group",
					inline = true,
					name = "Level Restriction",
					desc = "",
					order = 5,
					args = {
						enableLevelRestriction = {
							type = "toggle",
							name = "Enable",
							desc = "Enable Level Restriction",
							get = function(info)
								return HeirloomQuestTracker.db.global.toonOptions.levelRestriction
							end,
							set = function(info, value)
								HeirloomQuestTracker.db.global.toonOptions.levelRestriction = value
							end,
							order = 1,
						},
						minimumLevelOption = {
							type = "range",
							name = "Minimum Level",
							desc = "Show Toons this level and higher.",
							step = 1, min = 1, max = 100,
							get = function(info)
								return HeirloomQuestTracker.db.global.toonOptions.minimumLevel
							end,
							set = function(info, value)
								HeirloomQuestTracker.db.global.toonOptions.minimumLevel = value
							end,
							disabled = function()
								return not HeirloomQuestTracker.db.global.toonOptions.levelRestriction
							end,
							order = 2,
						},
					},
				},
				hideInactiveOptions = {
					type = "group",
					inline = true,
					name = "Hide Inactive Toons",
					desc = "",
					order = 6,
					args = {
						removeInactiveToons = {
							type = "toggle",
							name = "Enable",
							desc = "Enable hiding of inactive Toons.",
							get = function(info)
								return HeirloomQuestTracker.db.global.toonOptions.removeInactive
							end,
							set = function(info)
								HeirloomQuestTracker.db.global.toonOptions.removeInactive = value
							end,
							order = 1,
						},
						inactivityThresholdOption = {
							type = "range",
							name = "Iactivity Threshold in days",
							desc = "Hide a Toon after it has been inactive for X days.",
							step = 1, min = 7, max = 48,
							get = function(info)
								return HeirloomQuestTracker.db.global.toonOptions.inactivityThreshold
							end,
							set = function(info, value)
								HeirloomQuestTracker.db.global.toonOptions.inactivityThreshold = value
							end,
							disabled = function()
								return not HeirloomQuestTracker.db.global.toonOptions.removeInactive
							end,
							order = 2,
						},
					},
				},
				trackedToonOptions = {
					type = "group",
					inline = true,
					name = "Remove Tracked Toons",
					desc = "",
					order = 7,
					args = {
						serverSelect = {
							type = "select",
							name = "Server",
							desc = "Select Server to remove tracked Toon from.",
							values = function()
								local realmList = {}

								for realm in pairs(HeirloomQuestTracker.db.global.realms) do
									realmList[realm] = realm
								end

								return realmList
							end,
							get = function(info)
								return selectedRealm
							end,
							set = function(info, value)
								selectedRealm = value
								selectedToon = nil
							end,
							order = 1,
						},
						toonSelect = {
							type = "select",
							name = "Toon",
							desc = "Select tracked Toon to remove.",
							disabled = function()
								return selectedRealm == nil
							end,
							values = function()
								local list = {}
								local realmInfo = HeirloomQuestTracker.db.global.realms[selectedRealm]
								if realmInfo then
									local  toons = realmInfo.toons

									for key, value in pairs(toons) do
										list[key] = key
									end
								end
								return list
							end,
							get = function(info)
								return selectedToon
							end,
							set = function(info, value)
								selectedToon = value
							end,
							order = 2,
						},
						removeAction = {
							type = "execute",
							name = "Remove",
							desc = "Click to execute removale",
							disabled = function()
								return selectedRealm == nil or selectedToon == nil
							end,
							func = function ()
								local realmInfo = HeirloomQuestTracker.db.global.realms[selectedRealm]
								local toonInfo = realmInfo.toons[selectedToon]
								local count = 0

								if not realmInfo then
									return
								end

								if toonInfo then
									realmInfo.toons[selectedToon] = nil
								end

								for key, value in pairs(realmInfo.toons) do
									count = count + 1
								end

								if count == 0 then
									HeirloomQuestTracker.db.global.realms[selectedRealm] = nil
								end
							end,
							order = 3,
						},
					},
				},
			},
		},
	},	
}

local function CleanupToons()
	local threshold = HeirloomQuestTracker.db.global.toonOptions.inactivityThreshold * (24 * 60 * 60)

	if not HeirloomQuestTracker.db.global.toonOptions.removeInactive or threshold == 0 then
		return
	end

	for realm in pairs(HeirloomQuestTracker.db.global.realms) do
		local realmInfo = self.db.global.realms[realm]
		local toons = nil

		if realmInfo then
			local toons = realmInfo.toons

			for key, value in pairs(toons) do
				if value.lastUpdate and value.lastUpdate < time() - threshold then
					value = nil
				end
			end
		end
	end
end

function HeirloomQuestTracker:GetDentedCoins() 
	local itemCount = GetItemCount(coinID)
--	print("itemCount: " .. itemCount)
	
	if not dentedCoins then
		dentedCoins = {}
		dentedCoins.count = {}
	end

	dentedCoins.count = itemCount

	return dentedCoins
end

function HeirloomQuestTracker:DisplayToonInTooltip(toonName, toonInfo)
	local tooltip = HeirloomQuestTracker.tooltip
	local line = tooltip:AddLine()
	local factionIcon = ""
	local coins = 0
	local quest1 = ""
	local quest2 = ""
	local quest3 = ""
	local quest4 = ""

	if toonInfo.faction and toonInfo.faction == "Alliance" then
		factionIcon = textures.alliance
	elseif
		toonInfo.faction and toonInfo.faction == "Horde" then
		factionIcon = textures.horde
	end

	tooltip:SetCell(line, 2, factionIcon .. " " .. toonName .. " (" .. toonInfo.class .. ")")

	if (toonInfo.dentedCoins) then
		coins = toonInfo.dentedCoins.count
	end

	tooltip:SetCell(line, 3, coins)

	if (toonInfo.loomQuests) then
		quest1 = toonInfo.loomQuests.armorOne
		quest2 = toonInfo.loomQuests.armorTwo
		quest3 = toonInfo.loomQuests.weaponOne
		quest4 = toonInfo.loomQuests.weaponTwo
	end

	if quest1 == true then
		tooltip:SetCell(line, 4, textures.armorOne)
	else
		tooltip:SetCell(line, 4, textures.incomplete)
	end

	if quest2 == true then
		tooltip:SetCell(line, 5, textures.armorTwo)
	else
		tooltip:SetCell(line, 5, textures.incomplete)
	end

	if quest3 == true then
		tooltip:SetCell(line, 6, textures.weaponOne)
	else
		tooltip:SetCell(line, 6, textures.incomplete)
	end

	if quest4 == true then
		tooltip:SetCell(line, 7, textures.weaponTwo)
	else
		tooltip:SetCell(line, 7, textures.incomplete)
	end

	if toonInfo.class then
		local color = RAID_CLASS_COLORS[toonInfo.class]
		tooltip:SetCellTextColor(line, 2, color.r, color.g, color.b)
	end
end

function HeirloomQuestTracker:IsShowMinimapButton(info)
	return not self.db.global.MinimapButton.hide
end

function HeirloomQuestTracker:ToggleMinimapButton(info, value)
	self.db.global.MinimapButton.hide = not value

	if self.db.global.MinimapButton.hide then
		LDBIcon:Hide(addonName)
	else
		LDBIcon:Show(addonName)
	end

	LDBIcon:Refresh(addonName)
	LDBIcon:Refresh(addonName)
end

function HeirloomQuestTracker:ShowOptions()
	InterfaceOptionsFrame_OpenToCategory(self.optionsFrame)
	InterfaceOptionsFrame_OpenToCategory(self.optionsFrame)
end

function HeirloomQuestTracker:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("HeirloomQuestTrackerDB", defaults, true)

	LDBIcon:Register(addonName, HeirloomQuestTrackerLauncher, self.db.global.MinimapButton)

	local hqtcfg = LibStub("AceConfig-3.0")
	hqtcfg:RegisterOptionsTable("Heirloom Quest Tracker", options)
	hqtcfg:RegisterOptionsTable("Heirloom Quest Tracker General", options.args.generalOptions)
	hqtcfg:RegisterOptionsTable("Heirloom Quest Tracker Toons", options.args.toonOptions)

	local hqtdia = LibStub("AceConfigDialog-3.0")
	self.optionsFrame = hqtdia:AddToBlizOptions("Heirloom Quest Tracker", "Heirloom Quest Tracker")
	self.optionsFrame = LibStub:GetLibrary("LibAboutPanel").new("Heirloom Quest Tracker", "Heirloom Quest Tracker")
	hqtdia:AddToBlizOptions("Heirloom Quest Tracker General", "General Options", "Heirloom Quest Tracker")
	hqtdia:AddToBlizOptions("Heirloom Quest Tracker Toons", "Toons", "Heirloom Quest Tracker")
end

local function ShowHeader(tooltip, marker, headerName)
	line = tooltip:AddHeader()

	if (marker) then
		tooltip:SetCell(line, 1, marker)
	end

	tooltip:SetCell(line, 2, headerName, nil, nil, nil, nil, nil, 50)
	tooltip:SetCellTextColor(line, 2, yellow.r, yellow.g, yellow.b)

	tooltip:SetCell(line, 3, "|T" .. coinIcon .. ":0|t", nil, "CENTER")

	tooltip:SetCell(line, 4, "5 Coin Quest", "CENTER")
	tooltip:SetCellTextColor(line, 4, yellow.r, yellow.g, yellow.b)

	tooltip:SetCell(line, 5, "10 Coin Quest", "CENTER")
	tooltip:SetCellTextColor(line, 5, yellow.r, yellow.g, yellow.b)

	tooltip:SetCell(line, 6, "25 Coin Quest", "CENTER")
	tooltip:SetCellTextColor(line, 6, yellow.r, yellow.g, yellow.b)

	tooltip:SetCell(line, 7, "50 Coin Quest", "CENTER")
	tooltip:SetCellTextColor(line, 7, yellow.r, yellow.g, yellow.b)

	return line
end

function HeirloomQuestTracker:DisplayServerInTooltip(realmName)
	local realmInfo = self.db.global.realms[realmName]
	local toons = nil
	local collapsed = false
	local epoch = time() - (HeirloomQuestTracker.db.global.toonOptions.inactivityThreshold * 24 * 60 * 60)

	if realmInfo then
		toons = realmInfo.toons
		collapsed = realmInfo.collapsed
	end

	local toonNames = {}
	local currentToonName = UnitName("player")
	local currentRealmName = GetRealmName()
	local tooltip = HeirloomQuestTracker.tooltip
	local levelRestriction = HeirloomQuestTracker.db.global.toonOptions.levelRestriction or false;

	if HeirloomQuestTracker.db.global.toonOptions.levelRestriction then
		minimumLevel = HeirloomQuestTracker.db.global.toonOptions.minimumLevel
		if not minimumLevel then minimumLevel = 90 end
	end

	if not toons then
		return
	end

	if HeirloomQuestTracker.db.global.toonOptions.levelRestriction == true then
		for k, v in pairs(toons) do
			local include = true
			if (realmName ~= currentRealmName or k ~= currentToonName) and 
			(not HeirloomQuestTracker.db.global.toonOptions.removeInactive or v.lastUpdate > epoch) and 
			(v.level >= minimumLevel) then
				table.insert(toonNames, k);
			end
		end
	else
		for k, v in pairs(toons) do
			local include = true
			if (realmName ~= currentRealmName or k ~= currentToonName) and 
			(not HeirloomQuestTracker.db.global.toonOptions.removeInactive or v.lastUpdate > epoch) then
				table.insert(toonNames, k);
			end
		end
	end

	if (table.getn(toonNames) == 0) then
		return
	end

	table.sort(toonNames)

	tooltip:AddSeparator(2, 0, 0, 0, 0)

	if not collapsed then
		line = ShowHeader(tooltip, "|TInterface\\Buttons\\UI-MinusButton-Up:16|t", realmName)

		tooltip:AddSeparator(3, 0, 0, 0, 0)

		for k, v in pairs(toonNames) do
			HeirloomQuestTracker:DisplayToonInTooltip(v, toons[v])
		end

		tooltip:AddSeparator(1, 1, 1, 1, 1.0)
	else
		line = ShowHeader(tooltip, "|TInterface\\Buttons\\UI-PlusButton-Up:16|t", realmName)
	end

	tooltip:SetCellTextColor(line, 2, yellow.r, yellow.g, yellow.b)
	tooltip:SetCellScript(line, 1, "OnMouseUp", RealmOnClick, realmName)
end

function RealmOnClick(cell, realmName)
	HeirloomQuestTracker.db.global.realms[realmName].collapsed = not HeirloomQuestTracker.db.global.realms[realmName].collapsed
	HeirloomQuestTracker:ShowToolTip()
end

function HeirloomQuestTracker:ShowToolTip()
	local tooltip = HeirloomQuestTracker.tooltip
	local toonName = UnitName("player")
	local toons = HeirloomQuestTracker.db.realm.toons
	local class, className = UnitClass("player")
	local includeToons = HeirloomQuestTracker.db.global.toonOptions.include or 3

	local columnCount = 7

	RequestLFDPlayerLockInfo()

	if LibQTip:IsAcquired("HeirloomQuestTrackerTooltip") and tooltip then
		tooltip:Clear()
	else
		tooltip = LibQTip:Acquire("HeirloomQuestTrackerTooltip", columnCount, "CENTER", "LEFT", "CENTER", "CENTER", "CENTER", "CENTER", "CENTER")
		HeirloomQuestTracker.tooltip = tooltip
	end

	line = tooltip:AddHeader(" ")
	tooltip:SetCell(1, 1, "|T" .. coinIcon .. ":16|t" .. "|cff00ff00 Heirloom Quest Tracker|r", nil, "LEFT", tooltip:GetColumnCount())
	tooltip:AddSeparator(6, 0, 0, 0, 0)
	ShowHeader(tooltip, nil, "Toon")
	tooltip:AddSeparator(6, 0, 0, 0, 0)

	local info = HeirloomQuestTracker:GetToonInfo()
	HeirloomQuestTracker:DisplayToonInTooltip(toonName, info)
	tooltip:AddSeparator(6, 0, 0, 0, 0)
	tooltip:AddSeparator(1, 1, 1, 1, 1.0)

	if includeToons > 1 then
		HeirloomQuestTracker:DisplayServerInTooltip(GetRealmName())
	end

	if includeToons == 3 then
		realmNames = {}

		for k, v in pairs(HeirloomQuestTracker.db.global.realms) do
			if (k ~= GetRealmName()) then
				table.insert(realmNames, k);
			end
		end

		for k, v in pairs(realmNames) do
			HeirloomQuestTracker:DisplayServerInTooltip(v)
		end
	end

	line = tooltip:AddLine(" ")
	tooltip:SetCell(tooltip:GetLineCount(), 1, "Click to open options menu", nil, "LEFT", tooltip:GetColumnCount())

	if (frame) then
		tooltip:SetAutoHideDelay(0.01, frame)
		tooltip:SmartAnchorTo(frame)
	end

	tooltip:UpdateScrolling()
	tooltip:Show()
end

function HeirloomQuestTracker:SaveToonInfo()
	local toonName = UnitName("player")
	local realmName = GetRealmName()

	if not self.db.global.realms then
		self.db.global.realms = {}
	end

	local realmInfo = self.db.global.realms[realmName]

	if not realmInfo then
		realmInfo = {}
		realmInfo.toons = {}
	end

	realmInfo.toons[toonName] = HeirloomQuestTracker:GetToonInfo()

	self.db.global.realms[realmName] = realmInfo
end

function HeirloomQuestTracker:GetToonInfo()
	local toonInfo = {}
	local class, className = UnitClass("player")
	local level = UnitLevel("player")
	local englishFaction, localizedFaction = UnitFactionGroup("player")

	toonInfo.lastUpdate = time()
	toonInfo.class = className
	toonInfo.level = level
	toonInfo.faction = englishFaction
	toonInfo.dentedCoins = HeirloomQuestTracker:GetDentedCoins()
	toonInfo.loomQuests = HeirloomQuestTracker:GetQuestsDone(englishFaction)

	return toonInfo
end

function HeirloomQuestTracker:GetQuestsDone(englishFaction)
	local completed1 = ""
	local completed2 = ""
	local completed3 = ""
	local completed4 = ""

	-- QuestIDS
	local allianceQIDs = {
		38345,
		38394,
		38396,
		38402
	}
	local hordeQIDs = {
		38346,
		38395,
		38397,
		38404
	}
	local firstHeroicID = 37333


	if englishFaction == "Horde" then
		completed1 = IsQuestFlaggedCompleted(38346) -- 1 or nil edit: true or false
		
		completed2 = IsQuestFlaggedCompleted(38395)
		
		completed3 = IsQuestFlaggedCompleted(38397)
		
		completed4 = IsQuestFlaggedCompleted(38404)
	elseif englishFaction == "Alliance" then
		completed1 = IsQuestFlaggedCompleted(38345) -- 1 or nil edit: true or false

		completed2 = IsQuestFlaggedCompleted(38394)

		completed3 = IsQuestFlaggedCompleted(38396)

		completed4 = IsQuestFlaggedCompleted(38402)
	end

--	local isCompleted = IsQuestFlaggedCompleted(firstHeroicID)
--	print("isCompleted: " .. isCompleted)
	if not loomQuests then
		loomQuests = {}
		loomQuests.armorOne = ""
		loomQuests.armorTwo = ""
		loomQuests.weaponOne = ""
		loomQuests.weaponTwo = ""
	end

	loomQuests.armorOne = completed1
	loomQuests.armorTwo = completed2
	loomQuests.weaponOne = completed3
	loomQuests.weaponTwo = completed4

	return loomQuests
end

function HeirloomQuestTracker:UPDATE_INSTANCE_INFO()
	HeirloomQuestTracker:SaveToonInfo()
	if LibQTip:IsAcquired("WorldBossStatusTooltip") and HeirloomQuestTracker.tooltip then
		WorldBossStatus:ShowToolTip()
	end
end

function HeirloomQuestTracker:LFG_UPDATE_RANDOM_INFO()
	HeirloomQuestTracker:SaveToonInfo()
	if LibQTip:IsAcquired("WorldBossStatusTooltip") and HeirloomQuestTracker.tooltip then
		WorldBossStatus:ShowToolTip()
	end
end

function HeirloomQuestTracker:LFG_COMPLETION_REWARD()
	RequestLFDPlayerLockInfo()
end

function HeirloomQuestTracker:OnEnable()
	self:RegisterEvent("UPDATE_INSTANCE_INFO");
	self:RegisterEvent("LFG_UPDATE_RANDOM_INFO");
	self:RegisterEvent("LFG_COMPLETION_REWARD");	
end

function HeirloomQuestTracker:OnDisable()
	Self:UnregisterEvent("UPDATE_INSTANCE_INFO");
	Self:UnregisterEvent("LFG_UPDATE_RANDOM_INFO");
	Self:UnregisterEvent("LFG_COMPLETION_REWARD");
end

function SlashCmdList.HEIRLOOMQUESTTRACKER(msg)
	hqtUpdateData()
end